#include <cstdio>
#include <cstdlib>
#include <memory>
#include <iostream>

#include <ptlib.h>

// Undefine macros defined by ptlib that clash with other libraries
#undef _read
#undef _write
#undef _fdopen
#undef _lseek
#undef _close

#include <openh323/h323ep.h>

struct pt_process final :
    public PProcess
{
    pt_process() : PProcess("", "", 1, 0, PProcess::AlphaCode, 1, true) {}
    void Main() override {}
};

class endpoint final :
    public H323EndPoint
{
public:
    void init()
    {
        std::unique_ptr< H323ListenerTCP > listener(new H323ListenerTCP(*this, PIPSocket::Address("127.0.0.1"), 1920));
        if (StartListener(listener.get()))
        {
            listener.release();
            std::cout << "Listener started" << std::endl;
        }
        else
        {
            std::cout << "Listener could not be started" << std::endl;
        }
    }

    void terminate()
    {
        for (PINDEX idx = 0, n = listeners.GetSize(); idx < n; ++idx)
        {
            std::cout << "Stopping listener " << idx << std::endl;
            if (!listeners[idx].Close())
            {
                std::cout << "Cannot stop listener: " << listeners[idx] << std::endl;
            }
        }

        listeners.RemoveAll();

        std::cout << "All listeners stopped" << std::endl;
    }
};

int main()
{
    // Avoid loading any ptlib plugins
    setenv("PTLIBPLUGINDIR", "/dev/null", true);
    unsetenv("PWLIBPLUGINDIR");

    // ptlib requires this global object
    std::unique_ptr< pt_process > pprocess(new pt_process());

    std::unique_ptr< endpoint > ep(new endpoint());
    ep->init();

    std::cout << "Press Enter to terminate" << std::endl;
    std::getc(stdin);

    ep->terminate();
}
